procedure AskPortType( prompt : string; var s : str3 );
{ask user for a classification}
var
  ok : boolean;
  i : 1..3;
  ch : char;
begin
  write('Please enter a ');
  ok := false;
  repeat
    write('classification for the ', prompt, ' port: ');
    readln( s );
    if length( s ) <> 3 then
      begin
        writeln('Please enter a three letter string with S=sell, B=buy, X=don''t care');
        writeln('For example, selling fuel, don''t care about organic, buying equip is SXB');
      end
    else for i := 1 to 3 do if not (s[i] in ['b','B','s','S','x','X']) then
      begin
        write('I don''t understand ', s[i], ' for ');
        case i of
          1 : write('fuel ore');
          2 : write('organics');
          3 : write('equipment');
        end; {case}
        write(' Please try again.');
      end
    else
      ok := true;
  until ok;
  for i := 1 to 3 do
    s[i] := upcase(s[i]);
end;

function match( classification : str3; s : sector) : boolean;
{ return true when port in sector s matches the classification }
const
  powertwo : array [1..3] of integer = (1, 2, 4);
var
  ThisPort : stuff;
  matchall : boolean;
  i        : 1..3;
begin
  matchall := false;
  ThisPort := space.sectors[s]^.porttype;
  if length( classification ) <> 3 then
    writeln('error -- wrong length of string passed to "match"')
  else if ThisPort = NotAPort then
    writeln('error -- matched called on sector that isn''t a port')
  else if ThisPort <> Class0 then
    begin
      matchall := true;
      for i := 1 to 3 do
        case classification[i] of
          'B' : matchall := matchall and ( ThisPort and powertwo[i] = 0);
          'S' : matchall := matchall and ( ThisPort and powertwo[i] <> 0);
          'X' : ;
        else
          writeln('error: bad character in classification passed to "match"');
        end; {case}
    end;
  match := matchall;
end;

procedure AddPortsInsideRadius( CurrentSector : sector;
                                matchtype     : str3;
                                HowFar        : integer;
                            var MasterList    : sectorList );
{ add all ports to "master list" that match the described "match type"
  within "How Far" of "Current Sector". }
var
  t : warpindex;
begin
  if space.sectors[ currentSector ]^.porttype <> NotAPort then
    if match( matchtype, CurrentSector ) then
      AddToList( MasterList, currentSector );
  if HowFar > 0 then {recurse}
    begin
      for t := 1 to space.sectors[ CurrentSector ]^.number do
        AddPortsInsideRadius( space.sectors[ CurrentSector ]^.data[ t ],
                              matchtype, howFar - 1, MasterList );
    end;
end;

function SearchForPair( ShowLevels   : boolean;
                         baseSector  : sector;
                         destination : str3;
                         logToFile   : boolean;
                     var logfile     : text;
                         limit       : integer ) : integer;
var
  s          : sector;
  FoundPorts : sectorlist;
  i          : sectorindex;
  lines      : integer;
  g, g1      : stuff;
  mss        : string;
begin
  FoundPorts.size := 0;
  lines := 0;
  g := space.sectors[baseSector]^.porttype;
  AddPortsInsideRadius( baseSector, destination, limit, FoundPorts );
  if FoundPorts.size > 0 then
    begin
      mss := 'Base point ' + str( baseSector, 4) + status( g );
      if space.sectors[ baseSector]^.etc and HasFighters <> Nothing then
        mss := mss + ' F';
      if space.sectors[ BaseSector]^.etc and SpaceLane <> Nothing then
        mss := mss + ' SL';
      mss := mss + DisplayPort(portNumber(baseSector));
      lines := lines + 1;
      writeln( mss );
      if logToFile then
        writeln( logfile, mss );
      for i := 1 to FoundPorts.size do
        begin
          s := FoundPorts.data[i];
          g1 := space.sectors[s]^.porttype;
          mss := 'Pair:      ' + str( s , 4) + ' (';
          mss := mss + str(FixPath( s, baseSector, maxSector), 2) + ') ';
          mss := mss + status( g1 );
          if space.sectors[ s ]^.etc and HasFighters <> Nothing then
            mss := mss + ' F';
          mss := mss + ' '+ DisplayPort(portNumber(s));
          writeln( mss );
          if logToFile then
            writeln( logfile, mss );
          if ShowLevels then
            PortTradeFactor( baseSector, s, deal( g, g1), deal( g1, g ),
                             AllTrades, logToFile, logfile );
          lines := lines + 1;
        end;
    end; {if}
  SearchForPair := lines;
end;

procedure PairSearch( target, destination : str3 );
{ this will look for ports of type "target" (wildcards allowed) to type
  "destination".}
var
  linecount,              { number of lines printed this screen }
  lines      : integer;   { number of lines printed this search }
  NumSectors : sectorindex;
  maxDist    : integer;
  i          : sector;
  candidates : sectorlist;
  response   : string;
  showfactor : boolean;
  ch         : char;
  log        : text;
  logme      : boolean;
begin
  linecount := 0;
  SortPorts( NumSectors );
  candidates.size := 0;
  for i := 1 to NumSectors do
    if match( target, distances[i].s) then
      AddToList( candidates, distances[i].s );
  write('Maximum distance for pair? [0 to abort] ');
  maxDist := readNumberFromTerminal;
  showFactor := prompt('Show port trade factors? ');
  logme := LogToDisk( log,
        'Do you want an echo of the results to an ASCII file? ',
        BBSName+'.txt');

  for i := 1 to candidates.size do
    begin
      lines := SearchForPair( showfactor, candidates.data[i], destination,
                              logme, log, maxDist );
      if lines > 0 then
        begin
          linecount := linecount + lines;
          if showFactor then
            linecount := linecount + (lines - 1); {one factor level per line}
          if linecount > 10 then
            begin
              writeln;
              if prompt('enough? ') then
                begin
                  if logme then
                    close( log );
                  exit;
                end;
              linecount := 0;
            end {if linecount}
          else
            writeln;
        end; {if lines}
    end;  {if match}
  if logme then
    close( log );
end;
